@@ -94,10 +94,9 @@ class NerBIO:
 
         # update args
         all_steps = train_size / self.model_args.get('train_batch_size')
+
         self.model_args.update(
             {
-                'train_size': train_size,
-                'eval_size': eval_size,
                 'logging_steps': int(max(all_steps / 10 / self.model_args.get('gradient_accumulation_steps'), 1)),
                 'evaluate_during_training_steps': int(
                     max(all_steps / 10 / self.model_args.get('gradient_accumulation_steps'), 1)),
@@ -112,19 +111,19 @@ class NerBIO:
         # get model
         model = NERModel(model_type=self.model_type, model_name=self.pretrained_model, labels=self.labels,
                          args=self.model_args, use_cuda=self.use_cuda, cuda_device=self.cuda_device)
-
         # train
         try:
             start_time = time.time()
             logger.info(f'start training: model_version---{self.model_version}')
             model.train_model(train_data=train_data, eval_data=eval_data)
             logger.info('training finished!!!')
+            wandb.log( {'train_size': train_size, 'eval_size': eval_size,})
             end_time = time.time()
             logger.info(f'train time: {round(end_time - start_time, 4)} s')
         except Exception as error:
             logger.error(f'train failed!!! ERROR:{error}')
-        finally:
-            wandb.finish()
+        # finally:
+            # wandb.finish()
             # ModelUtils.remove_some_model_files(model.args)
 
     @staticmethod
