@@ -1,10 +1,3 @@
-# encoding: utf-8
-"""
-@author: zyl
-@file: t5.py
-@time: 2021/11/29 14:46
-@desc:
-"""
 import time
 
 import pandas as pd
@@ -41,6 +34,7 @@ class NerT5:
     def my_config(self):
         return {
             'train_batch_size': 8,
+            'max_seq_length': 256,
 
             # multiprocess
             'use_multiprocessing': False,
@@ -94,31 +88,56 @@ class NerT5:
             'top_p': None,
         }
 
-    @staticmethod
-    def _deal_with_df(df):
-        df = df[['prefix', 'input_text', 'target_text']]
-        df = df.astype('str')
-        return df
+    def _deal_with_df(self, data, sliding_window=False, delimiter='|'):
+        data = data[['prefix', 'input_text', 'target_text']]
+        data = data.astype('str')
 
-    def train(self, train_data: pd.DataFrame, eval_data: pd.DataFrame):
+        if sliding_window:
+            from transformers import T5Tokenizer
+            tokenizer = T5Tokenizer.from_pretrained(self.pretrained_model)
+            data['input_text'] = data['input_text'].apply(NerT5._split_text_with_sliding_window,
+                                                          args=(self.model_args.get('max_seq_length'),
+                                                                tokenizer, 0.8))
+            data = data.explode('input_text')
+
+            res = []
+            for i, t in zip(data['input_text'].tolist(), data['target_text'].tolist()):
+                if t != delimiter:
+                    all_entities = list(set(t.split(delimiter)))
+                    if '' in all_entities:
+                        all_entities.remove('')
+                    r = delimiter
+                    if all_entities:
+                        for e in all_entities:
+                            if str(e) in str(i):
+                                r = r + str(e) + delimiter
+                    res.append(r)
+                else:
+                    res.append(t)
+
+            data['target_text'] = res
+        data.dropna(inplace=True)
+        return data
+
+    def train(self, train_data: pd.DataFrame, eval_data: pd.DataFrame, sliding_window=False, wandb_log=None):
         # deal with dt
-        train_data = NerT5._deal_with_df(train_data)
-        eval_data = NerT5._deal_with_df(eval_data)
-        train_size = train_data.shape[0]
-        eval_size = eval_data.shape[0]
+        train_raw_size = train_data.shape[0]
+        eval_raw_size = eval_data.shape[0]
+        logger.info('processing data...')
+        train_data = self._deal_with_df(train_data, sliding_window=sliding_window, delimiter='|')
+        eval_data = self._deal_with_df(eval_data, sliding_window=sliding_window, delimiter='|')
 
+        train_size = train_data.shape[0]
         all_steps = train_size / self.model_args.get('train_batch_size')
         self.model_args.update(
             {
-                'train_size': train_size,
-                'eval_size': eval_size,
                 'logging_steps': int(max(all_steps / 10 / self.model_args.get('gradient_accumulation_steps'), 1)),
                 'evaluate_during_training_steps': int(
                     max(all_steps / 10 / self.model_args.get('gradient_accumulation_steps'), 1)),
                 'wandb_project': self.wandb_proj,
                 'wandb_kwargs': {
                     'name': self.model_version + time.strftime("_%m%d_%H:%M:%S", time.localtime()),
-                    'tags': [self.model_version, 'train']
+                    'tags': [self.model_version, 'train'],
                 }
             }
         )
@@ -129,9 +148,12 @@ class NerT5:
         # train
         try:
             start_time = time.time()
-            logger.info(f'start training: model_version---{self.model_version},train_size---{train_size}')
+            logger.info(f'start training: model_version---{self.model_version},train_size---{train_raw_size}')
             model.train_model(train_data=train_data, eval_data=eval_data)
             logger.info('training finished!!!')
+            wandb.log({"eval_size": eval_raw_size, 'train_size': train_raw_size})
+            if wandb_log:
+                wandb.log(wandb_log)
             end_time = time.time()
             logger.info(f'train time: {round(end_time - start_time, 4)} s')
         except Exception as error:
@@ -141,10 +163,10 @@ class NerT5:
             # ModelUtils.remove_some_model_files(model.args)
 
     def eval(self, eval_data: pd.DataFrame, check_in_input_text: bool = False, delimiter='|',
-             tokenizer=None, use_sliding_window=False, sliding_window=512, stride=0.8,
+             tokenizer=None, use_sliding_window=False, sliding_window=None, stride=0.8,
              pos_neg_ratio=None, use_multi_gpus=None, self_metric=False):
         # deal_with_dt
-        eval_data = NerT5._deal_with_df(eval_data)
+        eval_data = self._deal_with_df(eval_data, sliding_window=False)
         eval_size = eval_data.shape[0]
 
         # wand_b
@@ -164,8 +186,7 @@ class NerT5:
                                                       tokenizer=tokenizer, use_sliding_window=use_sliding_window,
                                                       sliding_window=sliding_window, stride=stride,
                                                       pos_neg_ratio=pos_neg_ratio, use_multi_gpus=use_multi_gpus,
-                                                      self_metric=self_metric
-                                                      )
+                                                      self_metric=self_metric)
 
             wandb_log = {"eval_size": eval_size}
             for k, v in eval_res.items():
@@ -240,13 +261,13 @@ class NerT5:
         print(f'sum report:')
         res_df = ner_metric.entity_recognition_v2(revised_target_texts, revised_pred_target_texts,
                                                   pos_neg_ratio=pos_neg_ratio, self_metric=self_metric)
-        eval_res['sum'] = res_df
+        eval_res['ner_t5_metric'] = res_df
 
         return eval_res  # {prefix:res_df},type:dict
 
     @staticmethod
     def predict_entity_recognition(model, prefixes: list, input_texts: list, use_sliding_window=False,
-                                   sliding_window=512, stride=0.8, tokenizer=None,
+                                   sliding_window=None, stride=0.8, tokenizer=None,
                                    delimiter='|', use_multi_gpus=None) -> list:
         """predict entity recognition in mt5 model,
         Args:
@@ -264,6 +285,9 @@ class NerT5:
         Returns:
             pred_target_texts:list,every element in pred_target_texts corresponds a prefix and an input_text
         """
+        if not sliding_window:
+            sliding_window = model.args.max_seq_length
+
         if len(input_texts) == 1:
             use_multi_gpus = None
         assert len(prefixes) == len(input_texts)
@@ -308,14 +332,15 @@ class NerT5:
         Returns:
             truncated_input_text: the list of truncated_input_text
         """
+        sliding_window = sliding_window - 8  # 防止一些词： <\s> <sep>等
 
         if not isinstance(text, str):
             text = str(text)
 
         if not tokenizer:
             try:
-                from transformers.models.t5 import T5Tokenizer
-                tokenizer = T5Tokenizer.from_pretrained("mt5-base")
+                from simpletransformers.t5 import T5Model
+                tokenizer = T5Model('mt5', 'google/mt5-base').tokenizer
             except Exception:
                 print('no tokenizer....')
 
@@ -366,7 +391,7 @@ class NerT5:
         if not tokenizer:
             try:
                 from transformers.models.t5 import T5Tokenizer
-                tokenizer = T5Tokenizer.from_pretrained("mt5-base")
+                tokenizer = T5Tokenizer.from_pretrained("google/mt5-base")
             except Exception:
                 print('no tokenizer....')
 
